@@ -28,12 +28,11 @@
 
 #include <compiz-plugin.h>
 
-#define CORE_ABIVERSION 20080828
+#define CORE_ABIVERSION 20091102
 
 #include <stdio.h>
 #include <sys/time.h>
 
-#include <X11/Xlib-xcb.h>
 #include <X11/Xutil.h>
 #include <X11/extensions/Xdamage.h>
 #include <X11/extensions/Xcomposite.h>
@@ -72,6 +71,7 @@ COMPIZ_BEGIN_DECLS
 typedef struct _CompTexture	  CompTexture;
 typedef struct _CompIcon	  CompIcon;
 typedef struct _CompWindowExtents CompWindowExtents;
+typedef struct _CompWindowExtents CompFullscreenMonitorSet;
 typedef struct _CompProgram	  CompProgram;
 typedef struct _CompFunction	  CompFunction;
 typedef struct _CompFunctionData  CompFunctionData;
@@ -220,6 +220,9 @@ extern Bool       noDetection;
 extern Bool	  useDesktopHints;
 extern Bool       onlyCurrentScreen;
 
+extern char	**initialPlugins;
+extern int 	nInitialPlugins;
+
 extern int  defaultRefreshRate;
 extern char *defaultTextureFilter;
 
@@ -746,65 +749,33 @@ removeFileWatch (CompFileWatchHandle handle);
 #define COMP_DISPLAY_OPTION_AUTORAISE_DELAY                  5
 #define COMP_DISPLAY_OPTION_CLOSE_WINDOW_KEY                 6
 #define COMP_DISPLAY_OPTION_CLOSE_WINDOW_BUTTON              7
-#define COMP_DISPLAY_OPTION_MAIN_MENU_KEY                    8
-#define COMP_DISPLAY_OPTION_RUN_DIALOG_KEY                   9
-#define COMP_DISPLAY_OPTION_COMMAND0                         10
-#define COMP_DISPLAY_OPTION_COMMAND1                         11
-#define COMP_DISPLAY_OPTION_COMMAND2                         12
-#define COMP_DISPLAY_OPTION_COMMAND3                         13
-#define COMP_DISPLAY_OPTION_COMMAND4                         14
-#define COMP_DISPLAY_OPTION_COMMAND5                         15
-#define COMP_DISPLAY_OPTION_COMMAND6                         16
-#define COMP_DISPLAY_OPTION_COMMAND7                         17
-#define COMP_DISPLAY_OPTION_COMMAND8                         18
-#define COMP_DISPLAY_OPTION_COMMAND9                         19
-#define COMP_DISPLAY_OPTION_COMMAND10                        20
-#define COMP_DISPLAY_OPTION_COMMAND11                        21
-#define COMP_DISPLAY_OPTION_RUN_COMMAND0_KEY                 22
-#define COMP_DISPLAY_OPTION_RUN_COMMAND1_KEY                 23
-#define COMP_DISPLAY_OPTION_RUN_COMMAND2_KEY                 24
-#define COMP_DISPLAY_OPTION_RUN_COMMAND3_KEY                 25
-#define COMP_DISPLAY_OPTION_RUN_COMMAND4_KEY                 26
-#define COMP_DISPLAY_OPTION_RUN_COMMAND5_KEY                 27
-#define COMP_DISPLAY_OPTION_RUN_COMMAND6_KEY                 28
-#define COMP_DISPLAY_OPTION_RUN_COMMAND7_KEY                 29
-#define COMP_DISPLAY_OPTION_RUN_COMMAND8_KEY                 30
-#define COMP_DISPLAY_OPTION_RUN_COMMAND9_KEY                 31
-#define COMP_DISPLAY_OPTION_RUN_COMMAND10_KEY                32
-#define COMP_DISPLAY_OPTION_RUN_COMMAND11_KEY                33
-#define COMP_DISPLAY_OPTION_SLOW_ANIMATIONS_KEY              34
-#define COMP_DISPLAY_OPTION_RAISE_WINDOW_KEY                 35
-#define COMP_DISPLAY_OPTION_RAISE_WINDOW_BUTTON              36
-#define COMP_DISPLAY_OPTION_LOWER_WINDOW_KEY                 37
-#define COMP_DISPLAY_OPTION_LOWER_WINDOW_BUTTON              38
-#define COMP_DISPLAY_OPTION_UNMAXIMIZE_WINDOW_KEY            39
-#define COMP_DISPLAY_OPTION_MINIMIZE_WINDOW_KEY              40
-#define COMP_DISPLAY_OPTION_MINIMIZE_WINDOW_BUTTON           41
-#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_KEY              42
-#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_HORZ_KEY         43
-#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_VERT_KEY         44
-#define COMP_DISPLAY_OPTION_SCREENSHOT                       45
-#define COMP_DISPLAY_OPTION_RUN_SCREENSHOT_KEY               46
-#define COMP_DISPLAY_OPTION_WINDOW_SCREENSHOT                47
-#define COMP_DISPLAY_OPTION_RUN_WINDOW_SCREENSHOT_KEY        48
-#define COMP_DISPLAY_OPTION_WINDOW_MENU_BUTTON               49
-#define COMP_DISPLAY_OPTION_WINDOW_MENU_KEY                  50
-#define COMP_DISPLAY_OPTION_SHOW_DESKTOP_KEY                 51
-#define COMP_DISPLAY_OPTION_SHOW_DESKTOP_EDGE                52
-#define COMP_DISPLAY_OPTION_RAISE_ON_CLICK                   53
-#define COMP_DISPLAY_OPTION_AUDIBLE_BELL                     54
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_KEY      55
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_BUTTON   56
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_HORZ_KEY 57
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_VERT_KEY 58
-#define COMP_DISPLAY_OPTION_HIDE_SKIP_TASKBAR_WINDOWS        59
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_SHADED_KEY         60
-#define COMP_DISPLAY_OPTION_IGNORE_HINTS_WHEN_MAXIMIZED      61
-#define COMP_DISPLAY_OPTION_TERMINAL			     62
-#define COMP_DISPLAY_OPTION_RUN_TERMINAL_KEY		     63
-#define COMP_DISPLAY_OPTION_PING_DELAY			     64
-#define COMP_DISPLAY_OPTION_EDGE_DELAY                       65
-#define COMP_DISPLAY_OPTION_NUM				     66
+#define COMP_DISPLAY_OPTION_SLOW_ANIMATIONS_KEY              8
+#define COMP_DISPLAY_OPTION_RAISE_WINDOW_KEY                 9
+#define COMP_DISPLAY_OPTION_RAISE_WINDOW_BUTTON              10
+#define COMP_DISPLAY_OPTION_LOWER_WINDOW_KEY                 11
+#define COMP_DISPLAY_OPTION_LOWER_WINDOW_BUTTON              12
+#define COMP_DISPLAY_OPTION_UNMAXIMIZE_WINDOW_KEY            13
+#define COMP_DISPLAY_OPTION_MINIMIZE_WINDOW_KEY              14
+#define COMP_DISPLAY_OPTION_MINIMIZE_WINDOW_BUTTON           15
+#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_KEY              16
+#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_HORZ_KEY         17
+#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_VERT_KEY         18
+#define COMP_DISPLAY_OPTION_WINDOW_MENU_BUTTON               19
+#define COMP_DISPLAY_OPTION_WINDOW_MENU_KEY                  20
+#define COMP_DISPLAY_OPTION_SHOW_DESKTOP_KEY                 21
+#define COMP_DISPLAY_OPTION_SHOW_DESKTOP_EDGE                22
+#define COMP_DISPLAY_OPTION_RAISE_ON_CLICK                   23
+#define COMP_DISPLAY_OPTION_AUDIBLE_BELL                     24
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_KEY      25
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_BUTTON   26
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_HORZ_KEY 27
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_VERT_KEY 28
+#define COMP_DISPLAY_OPTION_HIDE_SKIP_TASKBAR_WINDOWS        29
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_SHADED_KEY         30
+#define COMP_DISPLAY_OPTION_IGNORE_HINTS_WHEN_MAXIMIZED      31
+#define COMP_DISPLAY_OPTION_PING_DELAY			     32
+#define COMP_DISPLAY_OPTION_EDGE_DELAY                       33
+#define COMP_DISPLAY_OPTION_NUM				     34
 
 typedef void (*HandleEventProc) (CompDisplay *display,
 				 XEvent	     *event);
@@ -896,8 +867,6 @@ struct _CompDisplay {
 
     CompDisplay *next;
 
-    xcb_connection_t *connection;
-
     Display    *display;
     CompScreen *screens;
 
@@ -1016,10 +985,12 @@ struct _CompDisplay {
     Atom wmDeleteWindowAtom;
     Atom wmTakeFocusAtom;
     Atom wmPingAtom;
-    Atom wmSyncRequestAtom;
 
+    Atom wmSyncRequestAtom;
     Atom wmSyncRequestCounterAtom;
 
+    Atom wmFullscreenMonitorsAtom;
+
     Atom closeWindowAtom;
     Atom wmMoveResizeAtom;
     Atom moveResizeWindowAtom;
@@ -1030,8 +1001,6 @@ struct _CompDisplay {
     Atom xBackgroundAtom[2];
 
     Atom toolkitActionAtom;
-    Atom toolkitActionMainMenuAtom;
-    Atom toolkitActionRunDialogAtom;
     Atom toolkitActionWindowMenuAtom;
     Atom toolkitActionForceQuitDialogAtom;
 
@@ -1059,6 +1028,7 @@ struct _CompDisplay {
     GLenum textureFilter;
 
     Window activeWindow;
+    Window nextActiveWindow;
 
     Window below;
     char   displayString[256];
@@ -1091,6 +1061,8 @@ struct _CompDisplay {
     MatchPropertyChangedProc   matchPropertyChanged;
 
     LogMessageProc logMessage;
+
+    void *reserved;
 };
 
 #define GET_CORE_DISPLAY(object) ((CompDisplay *) (object))
@@ -1397,10 +1369,10 @@ typedef void (*ApplyScreenTransformProc) (CompScreen		  *screen,
 					  CompOutput		  *output,
 					  CompTransform	          *transform);
 
-typedef void (*EnableOutputClippingProc) (CompScreen 	      *screen,
+typedef void (*EnableOutputClippingProc) (CompScreen	      *screen,
 					  const CompTransform *transform,
 					  Region	      region,
-					  CompOutput 	      *output);
+					  CompOutput	      *output);
 
 typedef void (*DisableOutputClippingProc) (CompScreen *screen);
 
@@ -1539,7 +1511,7 @@ void
 enableOutputClipping (CompScreen	  *screen,
 		      const CompTransform *transform,
 		      Region		  region,
-		      CompOutput 	  *output);
+		      CompOutput	  *output);
 
 void
 disableOutputClipping (CompScreen *screen);
@@ -1791,6 +1763,8 @@ typedef GLXPixmap (*GLXCreatePixmapProc) (Display     *display,
 					  GLXFBConfig config,
 					  Pixmap      pixmap,
 					  const int   *attribList);
+typedef void (*GLXDestroyPixmapProc) (Display   *display,
+				      GLXPixmap pixmap);
 
 typedef void (*GLActiveTextureProc) (GLenum texture);
 typedef void (*GLClientActiveTextureProc) (GLenum texture);
@@ -1882,10 +1856,11 @@ typedef void (*WindowMoveNotifyProc) (CompWindow *window,
 				      int	 dy,
 				      Bool	 immediate);
 
-#define CompWindowGrabKeyMask    (1 << 0)
-#define CompWindowGrabButtonMask (1 << 1)
-#define CompWindowGrabMoveMask   (1 << 2)
-#define CompWindowGrabResizeMask (1 << 3)
+#define CompWindowGrabKeyMask         (1 << 0)
+#define CompWindowGrabButtonMask      (1 << 1)
+#define CompWindowGrabMoveMask        (1 << 2)
+#define CompWindowGrabResizeMask      (1 << 3)
+#define CompWindowGrabExternalAppMask (1 << 4)
 
 typedef void (*WindowGrabNotifyProc) (CompWindow   *window,
 				      int	   x,
@@ -1900,6 +1875,10 @@ typedef void (*WindowStateChangeNotifyProc) (CompWindow   *window,
 
 typedef void (*OutputChangeNotifyProc) (CompScreen *screen);
 
+typedef unsigned int (*AddSupportedAtomsProc) (CompScreen   *s,
+					       Atom         *atoms,
+					       unsigned int size);
+
 typedef void (*InitWindowWalkerProc) (CompScreen *screen,
 				      CompWalker *walker);
 
@@ -2161,6 +2140,7 @@ struct _CompScreen {
     GLXGetFBConfigsProc      getFBConfigs;
     GLXGetFBConfigAttribProc getFBConfigAttrib;
     GLXCreatePixmapProc      createPixmap;
+    GLXDestroyPixmapProc     destroyPixmap;
 
     GLActiveTextureProc       activeTexture;
     GLClientActiveTextureProc clientActiveTexture;
@@ -2219,8 +2199,11 @@ struct _CompScreen {
     WindowStateChangeNotifyProc windowStateChangeNotify;
 
     OutputChangeNotifyProc outputChangeNotify;
+    AddSupportedAtomsProc  addSupportedAtoms;
 
     InitWindowWalkerProc initWindowWalker;
+
+    void *reserved;
 };
 
 #define GET_CORE_SCREEN(object) ((CompScreen *) (object))
@@ -2275,6 +2258,9 @@ void
 setCurrentOutput (CompScreen *s,
 		  int	     outputNum);
 
+void
+setSupportedWmHints (CompScreen *s);
+
 void
 updateScreenBackground (CompScreen  *screen,
 			CompTexture *texture);
@@ -2541,6 +2527,12 @@ typedef enum {
     CompStackingUpdateModeInitialMapDeniedFocus
 } CompStackingUpdateMode;
 
+typedef enum {
+    CompFocusAllowed = 0,
+    CompFocusPrevent,
+    CompFocusDenied
+} CompFocusResult;
+
 struct _CompWindowExtents {
     int left;
     int right;
@@ -2575,6 +2567,7 @@ struct _CompWindow {
     int		      serverBorderWidth;
     Window	      transientFor;
     Window	      clientLeader;
+    XWMHints	      *hints;
     XSizeHints	      sizeHints;
     Pixmap	      pixmap;
     CompTexture       *texture;
@@ -2598,6 +2591,7 @@ struct _CompWindow {
     Bool	      damaged;
     Bool	      redirected;
     Bool	      managed;
+    Bool	      unmanaging;
     Bool	      bindFailed;
     Bool	      overlayWindow;
     int		      destroyRefCnt;
@@ -2646,6 +2640,9 @@ struct _CompWindow {
     XRectangle iconGeometry;
     Bool       iconGeometrySet;
 
+    XRectangle fullscreenMonitorRect;
+    Bool       fullscreenMonitorsSet;
+
     XWindowChanges saveWc;
     int		   saveMask;
 
@@ -2681,6 +2678,8 @@ struct _CompWindow {
 
     /* must be set by addWindowGeometry */
     DrawWindowGeometryProc drawWindowGeometry;
+
+    void *reserved;
 };
 
 #define GET_CORE_WINDOW(object) ((CompWindow *) (object))
@@ -2823,6 +2822,10 @@ setWmState (CompDisplay *display,
 	    int		state,
 	    Window      id);
 
+void
+setWindowFullscreenMonitors (CompWindow               *w,
+			     CompFullscreenMonitorSet *monitors);
+
 void
 setWindowFrameExtents (CompWindow	 *w,
 		       CompWindowExtents *input);
@@ -2878,7 +2881,8 @@ unsigned int
 adjustConfigureRequestForGravity (CompWindow     *w,
 				  XWindowChanges *xwc,
 				  unsigned int   xwcm,
-				  int            gravity);
+				  int            gravity,
+				  int            direction);
 
 void
 moveResizeWindow (CompWindow     *w,
@@ -3046,7 +3050,7 @@ void
 setWindowUserTime (CompWindow *w,
 		   Time       time);
 
-Bool
+CompFocusResult
 allowWindowFocus (CompWindow   *w,
 		  unsigned int noFocusMask,
 		  unsigned int viewportX,
