@@ -765,20 +765,6 @@ KWD::Decorator::x11EventFilter (XEvent *xevent)
 		    client->showWindowMenu (pos);
 		}
 	    }
-	    else if (action == Atoms::toolkitActionMainMenuAtom)
-	    {
-		dcopClient ()->send ("kicker",
-				     "kicker",
-				     "showKMenu()",
-				     QByteArray ());
-	    }
-	    else if (action == Atoms::toolkitActionRunDialogAtom)
-	    {
-		dcopClient ()->send ("kdesktop",
-				     "KDesktopIface",
-				     "popupExecuteCommand()",
-				     QByteArray ());
-	    }
 	    else if (action == Atoms::toolkitActionForceQuitDialogAtom)
 	    {
 		if (mClients.contains (xevent->xclient.window))
