@@ -460,7 +460,7 @@ annoInitiate (CompDisplay     *d,
     {
 	ANNO_SCREEN (s);
 
-	if (otherScreenGrabExist (s, 0))
+	if (otherScreenGrabExist (s, NULL))
 	    return FALSE;
 
 	if (!as->grabIndex)
@@ -529,7 +529,7 @@ annoEraseInitiate (CompDisplay     *d,
     {
 	ANNO_SCREEN (s);
 
-	if (otherScreenGrabExist (s, 0))
+	if (otherScreenGrabExist (s, NULL))
 	    return FALSE;
 
 	if (!as->grabIndex)
@@ -890,8 +890,9 @@ annoGetObjectOptions (CompPlugin *plugin,
 	(GetPluginObjectOptionsProc) annoGetDisplayOptions
     };
 
+    *count = 0;
     RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
-		     (void *) (*count = 0), (plugin, object, count));
+		     (void *) count, (plugin, object, count));
 }
 
 static CompBool
