@@ -113,6 +113,7 @@ typedef struct _DecorDisplay {
     DecorTexture	     *textures;
     Atom		     supportingDmCheckAtom;
     Atom		     winDecorAtom;
+    Atom		     requestFrameExtentsAtom;
     Atom		     decorAtom[DECOR_NUM];
 
     CompOption opt[DECOR_DISPLAY_OPTION_NUM];
@@ -128,6 +129,7 @@ typedef struct _DecorScreen {
     DrawWindowProc		  drawWindow;
     DamageWindowRectProc	  damageWindowRect;
     GetOutputExtentsForWindowProc getOutputExtentsForWindow;
+    AddSupportedAtomsProc         addSupportedAtoms;
 
     WindowMoveNotifyProc   windowMoveNotify;
     WindowResizeNotifyProc windowResizeNotify;
@@ -413,9 +415,15 @@ decorCreateDecoration (CompScreen *screen,
 				 XA_INTEGER, &actual, &format,
 				 &n, &nleft, &data);
 
-    if (result != Success || !n || !data)
+    if (result != Success || !data)
 	return NULL;
 
+    if (!n)
+    {
+	XFree (data);
+	return NULL;
+    }
+
     prop = (long *) data;
 
     if (decor_property_get_version (prop) != decor_version ())
@@ -742,6 +750,9 @@ decorWindowUpdate (CompWindow *w,
 	break;
     }
 
+    if (w->wmType & (CompWindowTypeDockMask | CompWindowTypeDesktopMask))
+	decorate = FALSE;
+
     if (w->attrib.override_redirect)
 	decorate = FALSE;
 
@@ -878,19 +889,23 @@ decorCheckForDmOnScreen (CompScreen *s,
 				 XA_WINDOW, &actual, &format,
 				 &n, &left, &data);
 
-    if (result == Success && n && data)
+    if (result == Success && data)
     {
-	XWindowAttributes attr;
+	if (n)
+	{
+	    XWindowAttributes attr;
+
+	    memcpy (&dmWin, data, sizeof (Window));
+
+	    compCheckForError (d->display);
+
+	    XGetWindowAttributes (d->display, dmWin, &attr);
+
+	    if (compCheckForError (d->display))
+		dmWin = None;
+	}
 
-	memcpy (&dmWin, data, sizeof (Window));
 	XFree (data);
-
-	compCheckForError (d->display);
-
-	XGetWindowAttributes (d->display, dmWin, &attr);
-
-	if (compCheckForError (d->display))
-	    dmWin = None;
     }
 
     if (dmWin != ds->dmWin)
@@ -962,6 +977,14 @@ decorHandleEvent (CompDisplay *d,
 	if (w)
 	    decorWindowUpdate (w, TRUE);
 	break;
+    case ClientMessage:
+	if (event->xclient.message_type == dd->requestFrameExtentsAtom)
+	{
+	    w = findWindowAtDisplay (d, event->xclient.window);
+	    if (w)
+		decorWindowUpdate (w, TRUE);
+	}
+	break;
     default:
 	if (event->type == d->damageEvent + XDamageNotify)
 	{
@@ -1098,6 +1121,26 @@ decorDamageWindowRect (CompWindow *w,
     return status;
 }
 
+static unsigned int
+decorAddSupportedAtoms (CompScreen   *s,
+			Atom         *atoms,
+			unsigned int size)
+{
+    unsigned int count;
+
+    DECOR_DISPLAY (s->display);
+    DECOR_SCREEN (s);
+
+    UNWRAP (ds, s, addSupportedAtoms);
+    count = (*s->addSupportedAtoms) (s, atoms, size);
+    WRAP (ds, s, addSupportedAtoms, decorAddSupportedAtoms);
+
+    if (count < size)
+	atoms[count++] = dd->requestFrameExtentsAtom;
+
+    return count;
+}
+
 static void
 decorGetOutputExtentsForWindow (CompWindow	  *w,
 				CompWindowExtents *output)
@@ -1492,6 +1535,8 @@ decorInitDisplay (CompPlugin  *p,
 	XInternAtom (d->display, DECOR_NORMAL_ATOM_NAME, 0);
     dd->decorAtom[DECOR_ACTIVE] =
 	XInternAtom (d->display, DECOR_ACTIVE_ATOM_NAME, 0);
+    dd->requestFrameExtentsAtom =
+	XInternAtom (d->display, "_NET_REQUEST_FRAME_EXTENTS", 0);
 
     WRAP (dd, d, handleEvent, decorHandleEvent);
     WRAP (dd, d, matchPropertyChanged, decorMatchPropertyChanged);
@@ -1547,10 +1592,12 @@ decorInitScreen (CompPlugin *p,
     WRAP (ds, s, windowMoveNotify, decorWindowMoveNotify);
     WRAP (ds, s, windowResizeNotify, decorWindowResizeNotify);
     WRAP (ds, s, windowStateChangeNotify, decorWindowStateChangeNotify);
+    WRAP (ds, s, addSupportedAtoms, decorAddSupportedAtoms);
 
     s->base.privates[dd->screenPrivateIndex].ptr = ds;
 
     decorCheckForDmOnScreen (s, FALSE);
+    setSupportedWmHints (s);
 
     if (!ds->dmWin)
 	ds->decoratorStartHandle = compAddTimeout (0, -1,
@@ -1582,6 +1629,9 @@ decorFiniScreen (CompPlugin *p,
     UNWRAP (ds, s, windowMoveNotify);
     UNWRAP (ds, s, windowResizeNotify);
     UNWRAP (ds, s, windowStateChangeNotify);
+    UNWRAP (ds, s, addSupportedAtoms);
+
+    setSupportedWmHints (s);
 
     free (ds);
 }
@@ -1673,8 +1723,9 @@ decorGetObjectOptions (CompPlugin *plugin,
 	(GetPluginObjectOptionsProc) decorGetDisplayOptions
     };
 
+    *count = 0;
     RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
-		     (void *) (*count = 0), (plugin, object, count));
+		     (void *) count, (plugin, object, count));
 }
 
 static CompBool
