@@ -76,6 +76,8 @@ typedef struct _MinWindow {
     Region region;
 
     int unmapCnt;
+
+    Bool ignoreDamage;
 } MinWindow;
 
 #define GET_MIN_DISPLAY(d)					 \
@@ -185,23 +187,22 @@ minGetWindowState (CompWindow *w)
     int		  result, format;
     unsigned long n, left;
     unsigned char *data;
+    int           retval = WithdrawnState;
 
     result = XGetWindowProperty (w->screen->display->display, w->id,
 				 w->screen->display->wmStateAtom, 0L, 1L, FALSE,
 				 w->screen->display->wmStateAtom,
 				 &actual, &format, &n, &left, &data);
 
-    if (result == Success && n && data)
+    if (result == Success && data)
     {
-	int state;
+	if (n)
+	    memcpy (&retval, data, sizeof (int));
 
-	memcpy (&state, data, sizeof (int));
 	XFree ((void *) data);
-
-	return state;
     }
 
-    return WithdrawnState;
+    return retval;
 }
 
 static int
@@ -331,11 +332,13 @@ minPreparePaintScreen (CompScreen *s,
 		    {
 			mw->state = mw->newState;
 
+			mw->ignoreDamage = TRUE;
 			while (mw->unmapCnt)
 			{
 			    unmapWindow (w);
 			    mw->unmapCnt--;
 			}
+			mw->ignoreDamage = FALSE;
 		    }
 		}
 		else if (mw->region && w->damaged)
@@ -354,11 +357,13 @@ minPreparePaintScreen (CompScreen *s,
 			    {
 				mw->shade = 0;
 
+				mw->ignoreDamage = TRUE;
 				while (mw->unmapCnt)
 				{
 				    unmapWindow (w);
 				    mw->unmapCnt--;
 				}
+				mw->ignoreDamage = FALSE;
 			    }
 			}
 		    }
@@ -556,11 +561,13 @@ minHandleEvent (CompDisplay *d,
 	    if (mw->region)
 		w->height = 0;
 
+	    mw->ignoreDamage = TRUE;
 	    while (mw->unmapCnt)
 	    {
 		unmapWindow (w);
 		mw->unmapCnt--;
 	    }
+	    mw->ignoreDamage = FALSE;
 	}
 	break;
     case UnmapNotify:
@@ -671,6 +678,9 @@ minDamageWindowRect (CompWindow *w,
     MIN_SCREEN (w->screen);
     MIN_WINDOW (w);
 
+    if (mw->ignoreDamage)
+	return TRUE;
+
     if (initial)
     {
 	if (mw->state == IconicState)
@@ -891,6 +901,8 @@ minInitWindow (CompPlugin *p,
 
     mw->unmapCnt = 0;
 
+    mw->ignoreDamage = FALSE;
+
     if (w->state & CompWindowStateHiddenMask)
     {
 	if (w->shaded)
@@ -924,8 +936,10 @@ minFiniWindow (CompPlugin *p,
 {
     MIN_WINDOW (w);
 
+    mw->ignoreDamage = TRUE;
     while (mw->unmapCnt--)
 	unmapWindow (w);
+    mw->ignoreDamage = FALSE;
 
     if (mw->region)
 	XDestroyRegion (mw->region);
@@ -972,8 +986,9 @@ minGetObjectOptions (CompPlugin *plugin,
 	(GetPluginObjectOptionsProc) minGetScreenOptions
     };
 
+    *count = 0;
     RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
-		     (void *) (*count = 0), (plugin, object, count));
+		     (void *) count, (plugin, object, count));
 }
 
 static CompBool
