@@ -99,6 +99,12 @@ typedef struct _ResizeDisplay {
     int		 pointerDx;
     int		 pointerDy;
     KeyCode	 key[NUM_KEYS];
+
+    Region       constraintRegion;
+    int          inRegionStatus;
+    int          lastGoodHotSpotY;
+    int          lastGoodWidth;
+    int          lastGoodHeight;
 } ResizeDisplay;
 
 typedef struct _ResizeScreen {
@@ -307,6 +313,22 @@ resizeFinishResizing (CompDisplay *d)
     rd->w = NULL;
 }
 
+static Region
+resizeGetConstraintRegion (CompScreen *s)
+{
+    Region       region;
+    int          i;
+
+    region = XCreateRegion ();
+    if (!region)
+	return NULL;
+
+    for (i = 0; i < s->nOutputDev; i++)
+	XUnionRectWithRegion (&s->outputDev[i].workArea, region, region);
+
+    return region;
+}
+
 static Bool
 resizeInitiate (CompDisplay     *d,
 		CompAction      *action,
@@ -349,10 +371,10 @@ resizeInitiate (CompDisplay     *d,
 	}
 	else if (!mask)
 	{
-	    unsigned int sectorSizeX = w->serverWidth / 3;
-	    unsigned int sectorSizeY = w->serverHeight / 3;
-	    unsigned int posX        = x - w->serverX;
-	    unsigned int posY        = y - w->serverY;
+	    int sectorSizeX = w->serverWidth / 3;
+	    int sectorSizeY = w->serverHeight / 3;
+	    int posX        = x - w->serverX;
+	    int posY        = y - w->serverY;
 
 	    if (posX < sectorSizeX)
 		mask |= ResizeLeftMask;
@@ -365,12 +387,12 @@ resizeInitiate (CompDisplay     *d,
 		mask |= ResizeDownMask;
 
 	    /* if the pointer was in the middle of the window,
-	       do nothing */
+	       just prevent input to the window */
 	    if (!mask)
-		return FALSE;
+		return TRUE;
 	}
 
-	if (otherScreenGrabExist (w->screen, "resize", 0))
+	if (otherScreenGrabExist (w->screen, "resize", NULL))
 	    return FALSE;
 
 	if (rd->w)
@@ -456,13 +478,23 @@ resizeInitiate (CompDisplay     *d,
 
 	if (rs->grabIndex)
 	{
+	    unsigned int grabMask = CompWindowGrabResizeMask |
+				    CompWindowGrabButtonMask;
+	    Bool sourceExternalApp = getBoolOptionNamed (option, nOption,
+							 "external", FALSE);
+
+	    if (sourceExternalApp)
+		grabMask |= CompWindowGrabExternalAppMask;
+
 	    BoxRec box;
 
 	    rd->releaseButton = button;
 
-	    (w->screen->windowGrabNotify) (w, x, y, state,
-					   CompWindowGrabResizeMask |
-					   CompWindowGrabButtonMask);
+	    (w->screen->windowGrabNotify) (w, x, y, state, grabMask);
+
+	    if (d->opt[COMP_DISPLAY_OPTION_RAISE_ON_CLICK].value.b)
+		updateWindowAttributes (w,
+					CompStackingUpdateModeAboveFullscreen);
 
 	    /* using the paint rectangle is enough here
 	       as we don't have any stretch yet */
@@ -478,6 +510,26 @@ resizeInitiate (CompDisplay     *d,
 
 		warpPointer (w->screen, xRoot - pointerX, yRoot - pointerY);
 	    }
+
+	    if (rd->constraintRegion)
+		XDestroyRegion (rd->constraintRegion);
+
+	    if (sourceExternalApp)
+	    {
+		/* Prevent resizing beyond work area edges when resize is
+		   initiated externally (e.g. with window frame or menu)
+		   and not with a key (e.g. alt+button) */
+
+		rd->inRegionStatus   = RectangleOut;
+		rd->lastGoodHotSpotY = -1;
+		rd->lastGoodWidth    = w->serverWidth;
+		rd->lastGoodHeight   = w->serverHeight;
+		rd->constraintRegion = resizeGetConstraintRegion (w->screen);
+	    }
+	    else
+	    {
+		rd->constraintRegion = NULL;
+	    }
 	}
     }
 
@@ -666,7 +718,10 @@ resizeHandleMotionEvent (CompScreen *s,
     if (rs->grabIndex)
     {
 	BoxRec box;
-	int    w, h;
+	int    w, h;                    /* size of window contents */
+	int    wX, wY, wWidth, wHeight; /* rect. for window contents+borders */
+	int    i;
+	int    workAreaSnapDistance = 15;
 
 	RESIZE_DISPLAY (s->display);
 
@@ -773,6 +828,238 @@ resizeHandleMotionEvent (CompScreen *s,
 
 	constrainNewWindowSize (rd->w, w, h, &w, &h);
 
+	/* compute rect. for window + borders */
+	wWidth  = w + rd->w->input.left + rd->w->input.right;
+	wHeight = h + rd->w->input.top + rd->w->input.bottom;
+
+	if (rd->mask & ResizeLeftMask)
+	    wX = rd->savedGeometry.x + rd->savedGeometry.width -
+		 (w + rd->w->input.left);
+	else
+	    wX = rd->savedGeometry.x - rd->w->input.left;
+
+	if (rd->mask & ResizeUpMask)
+	    wY = rd->savedGeometry.y + rd->savedGeometry.height -
+		 (h + rd->w->input.top);
+	else
+	    wY = rd->savedGeometry.y - rd->w->input.top;
+
+	/* Check if resized edge(s) are near a work-area boundary */
+	for (i = 0; i < s->nOutputDev; i++)
+	{
+	    const XRectangle *workArea = &s->outputDev[i].workArea;
+
+	    /* if window and work-area intersect in x axis */
+	    if (wX + wWidth > workArea->x &&
+		wX < workArea->x + workArea->width)
+	    {
+		if (rd->mask & ResizeLeftMask)
+		{
+		    int dw = workArea->x - wX;
+
+		    if (0 < dw && dw < workAreaSnapDistance)
+		    {
+			w      -= dw;
+			wWidth -= dw;
+			wX     += dw;
+		    }
+		}
+		else if (rd->mask & ResizeRightMask)
+		{
+		    int dw = wX + wWidth - (workArea->x + workArea->width);
+
+		    if (0 < dw && dw < workAreaSnapDistance)
+		    {
+			w      -= dw;
+			wWidth -= dw;
+		    }
+		}
+	    }
+
+	    /* if window and work-area intersect in y axis */
+	    if (wY + wHeight > workArea->y &&
+		wY < workArea->y + workArea->height)
+	    {
+		if (rd->mask & ResizeUpMask)
+		{
+		    int dh = workArea->y - wY;
+
+		    if (0 < dh && dh < workAreaSnapDistance)
+		    {
+			h       -= dh;
+			wHeight -= dh;
+			wY      += dh;
+		    }
+		}
+		else if (rd->mask & ResizeDownMask)
+		{
+		    int dh = wY + wHeight - (workArea->y + workArea->height);
+
+		    if (0 < dh && dh < workAreaSnapDistance)
+		    {
+			h       -= dh;
+			wHeight -= dh;
+		    }
+		}
+	    }
+	}
+
+	if (rd->constraintRegion)
+	{
+	    int minWidth  = 50;
+	    int minHeight = 50;
+
+	    /* rect. for a minimal height window + borders
+	       (used for the constraining in X axis) */
+	    int minimalInputHeight = minHeight +
+		                     rd->w->input.top + rd->w->input.bottom;
+
+	    /* small hot-spot square (on window's corner or edge) that is to be
+	       constrained to the combined output work-area region */
+	    int x, y;
+	    int width = rd->w->input.top; /* square size = title bar height */
+	    int height = width;
+	    int status;
+
+	    /* compute x & y for constrained hot-spot rect */
+	    if (rd->mask & ResizeLeftMask)
+		x = wX;
+	    else if (rd->mask & ResizeRightMask)
+		x = wX + wWidth - width;
+	    else
+		x = MIN (MAX (xRoot, wX), wX + wWidth - width);
+
+	    if (rd->mask & ResizeUpMask)
+		y = wY;
+	    else if (rd->mask & ResizeDownMask)
+		y = wY + wHeight - height;
+	    else
+		y = MIN (MAX (yRoot, wY), wY + wHeight - height);
+
+	    status = XRectInRegion (rd->constraintRegion,
+				    x, y, width, height);
+
+	    /* only constrain movement if previous position was valid */
+	    if (rd->inRegionStatus == RectangleIn)
+	    {
+		int xStatus, yForXResize;
+		int nx = x;
+		int nw = w;
+		int nh = h;
+
+		if (rd->mask & (ResizeLeftMask | ResizeRightMask))
+		{
+		    xStatus = status;
+
+		    if (rd->mask & ResizeUpMask)
+			yForXResize = wY + wHeight - minimalInputHeight;
+		    else if (rd->mask & ResizeDownMask)
+			yForXResize = wY + minimalInputHeight - height;
+		    else
+			yForXResize = y;
+
+		    if (XRectInRegion (rd->constraintRegion,
+				       x, yForXResize,
+				       width, height) != RectangleIn)
+		    {
+			if (rd->lastGoodHotSpotY >= 0)
+			    yForXResize = rd->lastGoodHotSpotY;
+			else
+			    yForXResize = y;
+		    }
+		}
+		if (rd->mask & ResizeLeftMask)
+		{
+		    while ((nw > minWidth) && xStatus != RectangleIn)
+		    {
+			xStatus = XRectInRegion (rd->constraintRegion,
+						 nx, yForXResize, width, height);
+			if (xStatus != RectangleIn)
+			{
+			    nw--;
+			    nx++;
+			}
+		    }
+		    if (nw > minWidth)
+		    {
+			x = nx;
+			w = nw;
+		    }
+		}
+		else if (rd->mask & ResizeRightMask)
+		{
+		    while ((nw > minWidth) && xStatus != RectangleIn)
+		    {
+			xStatus = XRectInRegion (rd->constraintRegion,
+						 nx, yForXResize,
+						 width, height);
+			if (xStatus != RectangleIn)
+			{
+			    nw--;
+			    nx--;
+			}
+		    }
+		    if (nw > minWidth)
+		    {
+			x = nx;
+			w = nw;
+		    }
+		}
+
+		if (rd->mask & ResizeUpMask)
+		{
+		    while ((nh > minHeight) && status != RectangleIn)
+		    {
+			status = XRectInRegion (rd->constraintRegion,
+						x, y, width, height);
+			if (status != RectangleIn)
+			{
+			    nh--;
+			    y++;
+			}
+		    }
+		    if (nh > minHeight)
+			h = nh;
+		}
+		else if (rd->mask & ResizeDownMask)
+		{
+		    while ((nh > minHeight) && status != RectangleIn)
+		    {
+			status = XRectInRegion (rd->constraintRegion,
+						x, y, width, height);
+			if (status != RectangleIn)
+			{
+			    nh--;
+			    y--;
+			}
+		    }
+		    if (nh > minHeight)
+			h = nh;
+		}
+
+		if (((rd->mask & (ResizeLeftMask | ResizeRightMask)) &&
+		     xStatus == RectangleIn) ||
+		    ((rd->mask & (ResizeUpMask | ResizeDownMask)) &&
+		     status == RectangleIn))
+		{
+		    /* hot-spot inside work-area region, store good values */
+		    rd->lastGoodHotSpotY = y;
+		    rd->lastGoodWidth    = w;
+		    rd->lastGoodHeight   = h;
+		}
+		else
+		{
+		    /* failed to find a good hot-spot position, restore size */
+		    w = rd->lastGoodWidth;
+		    h = rd->lastGoodHeight;
+		}
+	    }
+	    else
+	    {
+		rd->inRegionStatus = status;
+	    }
+	}
+
 	if (rd->mode != RESIZE_MODE_NORMAL)
 	{
 	    if (rd->mode == RESIZE_MODE_STRETCH)
@@ -867,20 +1154,24 @@ resizeHandleEvent (CompDisplay *d,
 		w = findWindowAtDisplay (d, event->xclient.window);
 		if (w)
 		{
-		    CompOption o[6];
+		    CompOption o[7];
 		    int	       option;
 
 		    o[0].type    = CompOptionTypeInt;
 		    o[0].name    = "window";
 		    o[0].value.i = event->xclient.window;
 
+		    o[1].type    = CompOptionTypeBool;
+		    o[1].name    = "external";
+		    o[1].value.b = TRUE;
+
 		    if (event->xclient.data.l[2] == WmMoveResizeSizeKeyboard)
 		    {
 			option = RESIZE_DISPLAY_OPTION_INITIATE_KEY;
 
 			resizeInitiate (d, &rd->opt[option].value.action,
 					CompActionStateInitKey,
-					o, 1);
+					o, 2);
 		    }
 		    else
 		    {
@@ -907,31 +1198,31 @@ resizeHandleEvent (CompDisplay *d,
 			/* TODO: not only button 1 */
 			if (mods & Button1Mask)
 			{
-			    o[1].type	 = CompOptionTypeInt;
-			    o[1].name	 = "modifiers";
-			    o[1].value.i = mods;
-
 			    o[2].type	 = CompOptionTypeInt;
-			    o[2].name	 = "x";
-			    o[2].value.i = event->xclient.data.l[0];
+			    o[2].name	 = "modifiers";
+			    o[2].value.i = mods;
 
 			    o[3].type	 = CompOptionTypeInt;
-			    o[3].name	 = "y";
-			    o[3].value.i = event->xclient.data.l[1];
+			    o[3].name	 = "x";
+			    o[3].value.i = event->xclient.data.l[0];
 
 			    o[4].type	 = CompOptionTypeInt;
-			    o[4].name	 = "direction";
-			    o[4].value.i = mask[event->xclient.data.l[2]];
+			    o[4].name	 = "y";
+			    o[4].value.i = event->xclient.data.l[1];
 
 			    o[5].type	 = CompOptionTypeInt;
-			    o[5].name	 = "button";
-			    o[5].value.i = event->xclient.data.l[3] ?
+			    o[5].name	 = "direction";
+			    o[5].value.i = mask[event->xclient.data.l[2]];
+
+			    o[6].type	 = CompOptionTypeInt;
+			    o[6].name	 = "button";
+			    o[6].value.i = event->xclient.data.l[3] ?
 				event->xclient.data.l[3] : -1;
 
 			    resizeInitiate (d,
 					    &rd->opt[option].value.action,
 					    CompActionStateInitButton,
-					    o, 6);
+					    o, 7);
 
 			    resizeHandleMotionEvent (w->screen, xRoot, yRoot);
 			}
@@ -1031,7 +1322,7 @@ resizePaintRectangle (CompScreen              *s,
     glPushMatrix ();
 
     transformToScreenSpace (s, output, -DEFAULT_Z_CAMERA, &sTransform);
-    
+
     glLoadMatrixf (sTransform.m);
 
     glDisableClientState (GL_TEXTURE_COORD_ARRAY);
@@ -1286,6 +1577,8 @@ resizeInitDisplay (CompPlugin  *p,
 	rd->key[i] = XKeysymToKeycode (d->display,
 				       XStringToKeysym (rKeys[i].name));
 
+    rd->constraintRegion = NULL;
+
     WRAP (rd, d, handleEvent, resizeHandleEvent);
 
     d->base.privates[displayPrivateIndex].ptr = rd;
@@ -1305,6 +1598,9 @@ resizeFiniDisplay (CompPlugin  *p,
 
     compFiniDisplayOptions (d, rd->opt, RESIZE_DISPLAY_OPTION_NUM);
 
+    if (rd->constraintRegion)
+	XDestroyRegion (rd->constraintRegion);
+
     free (rd);
 }
 
@@ -1422,8 +1718,9 @@ resizeGetObjectOptions (CompPlugin *plugin,
 	(GetPluginObjectOptionsProc) resizeGetDisplayOptions
     };
 
+    *count = 0;
     RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
-		     (void *) (*count = 0), (plugin, object, count));
+		     (void *) count, (plugin, object, count));
 }
 
 static CompBool
