@@ -129,7 +129,8 @@ moveInputFocusToOtherWindow (CompWindow *w)
     Bool        focussedAny = FALSE;
 
     if (w->id != d->activeWindow && w->id != d->nextActiveWindow)
-	return;
+	if (d->activeWindow != None)
+	    return;
 
     if (w->transientFor && w->transientFor != s->root)
     {
@@ -1381,8 +1382,8 @@ handleEvent (CompDisplay *d,
 		    setWmState (d, WithdrawnState, w->id);
 
 		w->placed     = FALSE;
+		w->unmanaging = w->managed;
 		w->managed    = FALSE;
-		w->unmanaging = TRUE;
 	    }
 
 	    unmapWindow (w);
@@ -1643,6 +1644,8 @@ handleEvent (CompDisplay *d,
 	    w = findWindowAtDisplay (d, event->xproperty.window);
 	    if (w)
 	    {
+		s = w->screen;
+
 		if (w->startupId)
 		    free (w->startupId);
 		
@@ -1652,11 +1655,10 @@ handleEvent (CompDisplay *d,
 		{
 		    Time            timestamp = 0;
 		    int             vx, vy, x, y;
-		    CompScreen      *s = w->screen;
 		    CompFocusResult focus;
 
 		    w->initialTimestampSet = FALSE;
-		    applyStartupProperties (w->screen, w);
+		    applyStartupProperties (s, w);
 
 		    if (w->initialTimestampSet)
 			timestamp = w->initialTimestamp;
@@ -1676,7 +1678,7 @@ handleEvent (CompDisplay *d,
 					      timestamp);
 
 		    if (focus == CompFocusAllowed)
-			(*w->screen->activateWindow) (w);
+			(*s->activateWindow) (w);
 		}
 	    }
 	}
@@ -1692,7 +1694,7 @@ handleEvent (CompDisplay *d,
     case ClientMessage:
 	if (event->xclient.message_type == d->winActiveAtom)
 	{
-	    w = findWindowAtDisplay (d, event->xclient.window);
+	    w = findTopLevelWindowAtDisplay (d, event->xclient.window);
 	    if (w)
 	    {
 		CompFocusResult focus = CompFocusAllowed;
@@ -2042,11 +2044,11 @@ handleEvent (CompDisplay *d,
 		    XWindowChanges xwc;
 		    unsigned int   xwcm, source;
 
-		    /* adjust for gravity */
+		    /* adjust for gravity, but only for frame size */
 		    xwc.x      = w->serverX;
 		    xwc.y      = w->serverY;
-		    xwc.width  = w->serverWidth;
-		    xwc.height = w->serverHeight;
+		    xwc.width  = 0;
+		    xwc.height = 0;
 
 		    xwcm = adjustConfigureRequestForGravity (w, &xwc,
 							     CWX | CWY,
@@ -2208,10 +2210,26 @@ handleEvent (CompDisplay *d,
 
 		state &= ~CompWindowStateDemandsAttentionMask;
 		changeWindowState (w, state);
-
-		if (d->nextActiveWindow == event->xfocus.window)
-		    d->nextActiveWindow = None;
 	    }
+	    else
+	    {
+		d->activeWindow = None;
+
+		s = findScreenAtDisplay (d, event->xfocus.window);
+		if (s)
+		{
+		    if (event->xfocus.detail == NotifyDetailNone ||
+			(event->xfocus.mode == NotifyNormal &&
+			 event->xfocus.detail == NotifyInferior))
+		    {
+			/* we don't want the root window to get focus */
+			focusDefaultWindow (s);
+		    }
+		}
+	    }
+
+	    if (d->nextActiveWindow == event->xfocus.window)
+		d->nextActiveWindow = None;
 	}
 	break;
     case EnterNotify:
