@@ -255,6 +255,72 @@ setVirtualScreenSize (CompScreen *screen,
 		      int	 hsize,
 		      int	 vsize)
 {
+    /* if hsize or vsize is being reduced */
+    if (hsize < screen->hsize ||
+	vsize < screen->vsize)
+    {
+	CompWindow *w;
+	int        tx = 0;
+	int        ty = 0;
+
+	if (screen->x >= hsize)
+	    tx = screen->x - (hsize - 1);
+	if (screen->y >= vsize)
+	    ty = screen->y - (vsize - 1);
+
+	if (tx != 0 || ty != 0)
+	    moveScreenViewport (screen, tx, ty, TRUE);
+
+	/* Move windows that were in one of the deleted viewports into the
+	   closest viewport */
+	for (w = screen->windows; w; w = w->next)
+	{
+	    int moveX = 0;
+	    int moveY = 0;
+
+	    if (windowOnAllViewports (w))
+		continue;
+
+	    /* Find which viewport the (inner) window's top-left corner falls
+	       in, and check if it's outside the new viewport horizontal and
+	       vertical index range */
+	    if (hsize < screen->hsize)
+	    {
+		int vpX;   /* x index of a window's vp */
+
+		vpX = w->serverX / screen->width;
+		if (w->serverX < 0)
+		    vpX -= 1;
+
+		vpX += screen->x; /* Convert relative to absolute vp index */
+
+		/* Move windows too far right to left */
+		if (vpX >= hsize)
+		    moveX = ((hsize - 1) - vpX) * screen->width;
+	    }
+	    if (vsize < screen->vsize)
+	    {
+		int vpY;   /* y index of a window's vp */
+
+		vpY = w->serverY / screen->height;
+		if (w->serverY < 0)
+		    vpY -= 1;
+
+		vpY += screen->y; /* Convert relative to absolute vp index */
+
+		/* Move windows too far right to left */
+		if (vpY >= vsize)
+		    moveY = ((vsize - 1) - vpY) * screen->height;
+	    }
+
+	    if (moveX != 0 || moveY != 0)
+	    {
+		moveWindow (w, moveX, moveY, TRUE, TRUE);
+		syncWindowPosition (w);
+	    }
+	}
+    }
+
     screen->hsize = hsize;
     screen->vsize = vsize;
 
@@ -271,6 +337,7 @@ updateOutputDevices (CompScreen	*s)
     unsigned int  width, height;
     int		  x1, y1, x2, y2;
     Region	  region;
+    CompWindow    *w;
 
     for (i = 0; i < list->nValue; i++)
     {
@@ -373,6 +440,12 @@ updateOutputDevices (CompScreen	*s)
 
     setCurrentOutput (s, s->currentOutputDev);
 
+    /* clear out fullscreen monitor hints of all windows as
+       suggested on monitor layout changes in EWMH */
+    for (w = s->windows; w; w = w->next)
+	if (w->fullscreenMonitorsSet)
+	    setWindowFullscreenMonitors (w, NULL);
+
     updateWorkareaForScreen (s);
 
     setDefaultViewport (s);
@@ -697,9 +770,10 @@ addSequence (CompScreen        *screen,
     screen->startupSequences = s;
 
     if (!screen->startupSequenceTimeoutHandle)
-	compAddTimeout (1000, 1500,
-			startupSequenceTimeout,
-			screen);
+	screen->startupSequenceTimeoutHandle =
+	    compAddTimeout (1000, 1500,
+			    startupSequenceTimeout,
+			    screen);
 
     updateStartupFeedback (screen);
 }
@@ -739,6 +813,28 @@ removeSequence (CompScreen        *screen,
     updateStartupFeedback (screen);
 }
 
+static void
+removeAllSequences (CompScreen *screen)
+{
+    CompStartupSequence *s;
+    CompStartupSequence *sNext;
+
+    for (s = screen->startupSequences; s; s = sNext)
+    {
+	sNext = s->next;
+	sn_startup_sequence_unref (s->sequence);
+	free (s);
+    }
+    screen->startupSequences = NULL;
+
+    if (screen->startupSequenceTimeoutHandle)
+    {
+	compRemoveTimeout (screen->startupSequenceTimeoutHandle);
+	screen->startupSequenceTimeoutHandle = 0;
+    }
+    updateStartupFeedback (screen);
+}
+
 static void
 compScreenSnEvent (SnMonitorEvent *event,
 		   void           *userData)
@@ -905,7 +1001,7 @@ reshape (CompScreen *s,
     s->fullscreenOutput.workArea.y       = 0;
     s->fullscreenOutput.workArea.width   = w;
     s->fullscreenOutput.workArea.height  = h;
- 
+
     updateScreenEdges (s);
 }
 
@@ -977,7 +1073,7 @@ updateScreenBackground (CompScreen  *screen,
 				     &actualType, &actualFormat, &nItems,
 				     &bytesAfter, &prop);
 
-	if (status == Success && nItems && prop)
+	if (status == Success && prop)
 	{
 	    if (actualType   == pixmapAtom &&
 		actualFormat == 32         &&
@@ -1103,104 +1199,114 @@ setSupportingWmCheck (CompScreen *s)
 		     (unsigned char *) &s->grabWindow, 1);
 }
 
-static void
-setSupported (CompScreen *s)
+static unsigned int
+addSupportedAtoms (CompScreen   *s,
+		   Atom         *atoms,
+		   unsigned int size)
 {
-    CompDisplay *d = s->display;
-    Atom	data[256];
-    int		i = 0;
+    CompDisplay  *d = s->display;
+    unsigned int count = 0;
 
-    data[i++] = d->supportedAtom;
-    data[i++] = d->supportingWmCheckAtom;
+    atoms[count++] = d->utf8StringAtom;
 
-    data[i++] = d->utf8StringAtom;
+    atoms[count++] = d->clientListAtom;
+    atoms[count++] = d->clientListStackingAtom;
 
-    data[i++] = d->clientListAtom;
-    data[i++] = d->clientListStackingAtom;
+    atoms[count++] = d->winActiveAtom;
 
-    data[i++] = d->winActiveAtom;
+    atoms[count++] = d->desktopViewportAtom;
+    atoms[count++] = d->desktopGeometryAtom;
+    atoms[count++] = d->currentDesktopAtom;
+    atoms[count++] = d->numberOfDesktopsAtom;
+    atoms[count++] = d->showingDesktopAtom;
 
-    data[i++] = d->desktopViewportAtom;
-    data[i++] = d->desktopGeometryAtom;
-    data[i++] = d->currentDesktopAtom;
-    data[i++] = d->numberOfDesktopsAtom;
-    data[i++] = d->showingDesktopAtom;
+    atoms[count++] = d->workareaAtom;
 
-    data[i++] = d->workareaAtom;
+    atoms[count++] = d->wmNameAtom;
 
-    data[i++] = d->wmNameAtom;
-/*
-    data[i++] = d->wmVisibleNameAtom;
-*/
+    atoms[count++] = d->wmStrutAtom;
+    atoms[count++] = d->wmStrutPartialAtom;
 
-    data[i++] = d->wmStrutAtom;
-    data[i++] = d->wmStrutPartialAtom;
+    atoms[count++] = d->wmUserTimeAtom;
+    atoms[count++] = d->frameExtentsAtom;
+    atoms[count++] = d->frameWindowAtom;
 
-/*
-    data[i++] = d->wmPidAtom;
-*/
+    atoms[count++] = d->winStateAtom;
+    atoms[count++] = d->winStateModalAtom;
+    atoms[count++] = d->winStateStickyAtom;
+    atoms[count++] = d->winStateMaximizedVertAtom;
+    atoms[count++] = d->winStateMaximizedHorzAtom;
+    atoms[count++] = d->winStateShadedAtom;
+    atoms[count++] = d->winStateSkipTaskbarAtom;
+    atoms[count++] = d->winStateSkipPagerAtom;
+    atoms[count++] = d->winStateHiddenAtom;
+    atoms[count++] = d->winStateFullscreenAtom;
+    atoms[count++] = d->winStateAboveAtom;
+    atoms[count++] = d->winStateBelowAtom;
+    atoms[count++] = d->winStateDemandsAttentionAtom;
 
-    data[i++] = d->wmUserTimeAtom;
-    data[i++] = d->frameExtentsAtom;
-    data[i++] = d->frameWindowAtom;
-
-    data[i++] = d->winStateAtom;
-    data[i++] = d->winStateModalAtom;
-    data[i++] = d->winStateStickyAtom;
-    data[i++] = d->winStateMaximizedVertAtom;
-    data[i++] = d->winStateMaximizedHorzAtom;
-    data[i++] = d->winStateShadedAtom;
-    data[i++] = d->winStateSkipTaskbarAtom;
-    data[i++] = d->winStateSkipPagerAtom;
-    data[i++] = d->winStateHiddenAtom;
-    data[i++] = d->winStateFullscreenAtom;
-    data[i++] = d->winStateAboveAtom;
-    data[i++] = d->winStateBelowAtom;
-    data[i++] = d->winStateDemandsAttentionAtom;
-
-    data[i++] = d->winOpacityAtom;
-    data[i++] = d->winBrightnessAtom;
+    atoms[count++] = d->winOpacityAtom;
+    atoms[count++] = d->winBrightnessAtom;
 
     if (s->canDoSaturated)
     {
-	data[i++] = d->winSaturationAtom;
-	data[i++] = d->winStateDisplayModalAtom;
+	atoms[count++] = d->winSaturationAtom;
+	atoms[count++] = d->winStateDisplayModalAtom;
     }
 
-    data[i++] = d->wmAllowedActionsAtom;
+    atoms[count++] = d->wmAllowedActionsAtom;
 
-    data[i++] = d->winActionMoveAtom;
-    data[i++] = d->winActionResizeAtom;
-    data[i++] = d->winActionStickAtom;
-    data[i++] = d->winActionMinimizeAtom;
-    data[i++] = d->winActionMaximizeHorzAtom;
-    data[i++] = d->winActionMaximizeVertAtom;
-    data[i++] = d->winActionFullscreenAtom;
-    data[i++] = d->winActionCloseAtom;
-    data[i++] = d->winActionShadeAtom;
-    data[i++] = d->winActionChangeDesktopAtom;
-    data[i++] = d->winActionAboveAtom;
-    data[i++] = d->winActionBelowAtom;
+    atoms[count++] = d->winActionMoveAtom;
+    atoms[count++] = d->winActionResizeAtom;
+    atoms[count++] = d->winActionStickAtom;
+    atoms[count++] = d->winActionMinimizeAtom;
+    atoms[count++] = d->winActionMaximizeHorzAtom;
+    atoms[count++] = d->winActionMaximizeVertAtom;
+    atoms[count++] = d->winActionFullscreenAtom;
+    atoms[count++] = d->winActionCloseAtom;
+    atoms[count++] = d->winActionShadeAtom;
+    atoms[count++] = d->winActionChangeDesktopAtom;
+    atoms[count++] = d->winActionAboveAtom;
+    atoms[count++] = d->winActionBelowAtom;
 
-    data[i++] = d->winTypeAtom;
-    data[i++] = d->winTypeDesktopAtom;
-    data[i++] = d->winTypeDockAtom;
-    data[i++] = d->winTypeToolbarAtom;
-    data[i++] = d->winTypeMenuAtom;
-    data[i++] = d->winTypeSplashAtom;
-    data[i++] = d->winTypeDialogAtom;
-    data[i++] = d->winTypeUtilAtom;
-    data[i++] = d->winTypeNormalAtom;
+    atoms[count++] = d->winTypeAtom;
+    atoms[count++] = d->winTypeDesktopAtom;
+    atoms[count++] = d->winTypeDockAtom;
+    atoms[count++] = d->winTypeToolbarAtom;
+    atoms[count++] = d->winTypeMenuAtom;
+    atoms[count++] = d->winTypeSplashAtom;
+    atoms[count++] = d->winTypeDialogAtom;
+    atoms[count++] = d->winTypeUtilAtom;
+    atoms[count++] = d->winTypeNormalAtom;
 
-    data[i++] = d->wmDeleteWindowAtom;
-    data[i++] = d->wmPingAtom;
+    atoms[count++] = d->wmDeleteWindowAtom;
+    atoms[count++] = d->wmPingAtom;
 
-    data[i++] = d->wmMoveResizeAtom;
-    data[i++] = d->moveResizeWindowAtom;
-    data[i++] = d->restackWindowAtom;
+    atoms[count++] = d->wmMoveResizeAtom;
+    atoms[count++] = d->moveResizeWindowAtom;
+    atoms[count++] = d->restackWindowAtom;
+
+    atoms[count++] = d->wmFullscreenMonitorsAtom;
+
+    assert (count < size);
+
+    return count;
+}
+
+void
+setSupportedWmHints (CompScreen *s)
+{
+    CompDisplay  *d = s->display;
+    Atom	 data[256];
+    unsigned int count = 0;
+
+    data[count++] = d->supportedAtom;
+    data[count++] = d->supportingWmCheckAtom;
+
+    count += (*s->addSupportedAtoms) (s, data + count, 256 - count);
 
     XChangeProperty (d->display, s->root, d->supportedAtom, XA_ATOM, 32,
-		     PropModeReplace, (unsigned char *) data, i);
+		     PropModeReplace, (unsigned char *) data, count);
 }
 
 static void
@@ -1220,13 +1326,16 @@ getDesktopHints (CompScreen *s)
 				     XA_CARDINAL, &actual, &format,
 				     &n, &left, &propData);
 
-	if (result == Success && n && propData)
+	if (result == Success && propData)
 	{
-	    memcpy (data, propData, sizeof (unsigned long));
-	    XFree (propData);
+	    if (n)
+	    {
+		memcpy (data, propData, sizeof (unsigned long));
 
-	    if (data[0] > 0 && data[0] < 0xffffffff)
-		s->nDesktop = data[0];
+		if (data[0] > 0 && data[0] < 0xffffffff)
+		    s->nDesktop = data[0];
+	    }
+	    XFree (propData);
 	}
 
 	result = XGetWindowProperty (s->display->display, s->root,
@@ -1234,13 +1343,17 @@ getDesktopHints (CompScreen *s)
 				     XA_CARDINAL, &actual, &format,
 				     &n, &left, &propData);
 
-	if (result == Success && n && propData)
+	if (result == Success && propData)
 	{
-	    memcpy (data, propData, sizeof (unsigned long));
-	    XFree (propData);
+	    if (n)
+	    {
+		memcpy (data, propData, sizeof (unsigned long));
 
-	    if (data[0] < s->nDesktop)
-		s->currentDesktop = data[0];
+		if (data[0] < s->nDesktop)
+		    s->currentDesktop = data[0];
+	    }
+
+	    XFree (propData);
 	}
     }
 
@@ -1249,7 +1362,7 @@ getDesktopHints (CompScreen *s)
 				 FALSE, XA_CARDINAL, &actual, &format,
 				 &n, &left, &propData);
 
-    if (result == Success && n && propData)
+    if (result == Success && propData)
     {
 	if (n == 2)
 	{
@@ -1270,13 +1383,16 @@ getDesktopHints (CompScreen *s)
 				 XA_CARDINAL, &actual, &format,
 				 &n, &left, &propData);
 
-    if (result == Success && n && propData)
+    if (result == Success && propData)
     {
-	memcpy (data, propData, sizeof (unsigned long));
-	XFree (propData);
+	if (n)
+	{
+	    memcpy (data, propData, sizeof (unsigned long));
 
-	if (data[0])
-	    (*s->enterShowDesktopMode) (s);
+	    if (data[0])
+		(*s->enterShowDesktopMode) (s);
+	}
+	XFree (propData);
     }
 
     data[0] = s->currentDesktop;
@@ -1565,6 +1681,9 @@ freeScreen (CompScreen *s)
     if (s->grabs)
 	free (s->grabs);
 
+    if (s->exposeRects)
+	free (s->exposeRects);
+
     /* XXX: Maybe we should free all fragment functions here? But
        the definition of CompFunction is private to fragment.c ... */
     for (i = 0; i < 2; i++)
@@ -1780,6 +1899,7 @@ addScreen (CompDisplay *display,
     s->windowStateChangeNotify = windowStateChangeNotify;
 
     s->outputChangeNotify = outputChangeNotify;
+    s->addSupportedAtoms  = addSupportedAtoms;
 
     s->initWindowWalker = initWindowWalker;
 
@@ -1899,6 +2019,8 @@ addScreen (CompDisplay *display,
 	getProcAddress (s, "glXGetFBConfigAttrib");
     s->createPixmap = (GLXCreatePixmapProc)
 	getProcAddress (s, "glXCreatePixmap");
+    s->destroyPixmap = (GLXDestroyPixmapProc)
+	getProcAddress (s, "glXDestroyPixmap");
 
     if (!s->bindTexImage)
     {
@@ -1917,7 +2039,8 @@ addScreen (CompDisplay *display,
     if (!s->queryDrawable     ||
 	!s->getFBConfigs      ||
 	!s->getFBConfigAttrib ||
-	!s->createPixmap)
+	!s->createPixmap      ||
+	!s->destroyPixmap)
     {
 	compLogMessage ("core", CompLogLevelFatal,
 			"fbconfig functions missing");
@@ -1998,6 +2121,10 @@ addScreen (CompDisplay *display,
 	strstr (glExtensions, "GL_SGIS_texture_border_clamp"))
 	s->textureBorderClamp = 1;
 
+    s->activeTexture       = NULL;
+    s->clientActiveTexture = NULL;
+    s->multiTexCoord2f     = NULL;
+
     s->maxTextureUnits = 1;
     if (strstr (glExtensions, "GL_ARB_multitexture"))
     {
@@ -2012,6 +2139,14 @@ addScreen (CompDisplay *display,
 	    glGetIntegerv (GL_MAX_TEXTURE_UNITS_ARB, &s->maxTextureUnits);
     }
 
+    s->genPrograms             = NULL;
+    s->deletePrograms          = NULL;
+    s->bindProgram             = NULL;
+    s->programString           = NULL;
+    s->programEnvParameter4f   = NULL;
+    s->programLocalParameter4f = NULL;
+    s->getProgramiv            = NULL;
+
     s->fragmentProgram = 0;
     if (strstr (glExtensions, "GL_ARB_fragment_program"))
     {
@@ -2040,6 +2175,13 @@ addScreen (CompDisplay *display,
 	    s->fragmentProgram = 1;
     }
 
+    s->genFramebuffers        = NULL;
+    s->deleteFramebuffers     = NULL;
+    s->bindFramebuffer        = NULL;
+    s->checkFramebufferStatus = NULL;
+    s->framebufferTexture2D   = NULL;
+    s->generateMipmap         = NULL;
+
     s->fbo = 0;
     if (strstr (glExtensions, "GL_EXT_framebuffer_object"))
     {
@@ -2291,6 +2433,11 @@ addScreen (CompDisplay *display,
 	}
     }
 
+    /* enforce restack on all windows */
+    for (i = 0, w = s->reverseWindows; w && i < nchildren; i++, w = w->prev)
+	children[i] = w->id;
+    XRestackWindows (dpy, children, i);
+
     XFree (children);
 
     attrib.override_redirect = 1;
@@ -2327,7 +2474,7 @@ addScreen (CompDisplay *display,
 
     setDesktopHints (s);
     setSupportingWmCheck (s);
-    setSupported (s);
+    setSupportedWmHints (s);
 
     s->normalCursor = XCreateFontCursor (dpy, XC_left_ptr);
     s->busyCursor   = XCreateFontCursor (dpy, XC_watch);
@@ -2357,6 +2504,8 @@ removeScreen (CompScreen *s)
     else
 	d->screens = NULL;
 
+    removeAllSequences (s);
+
     while (s->windows)
 	removeWindow (s->windows);
 
@@ -2448,7 +2597,7 @@ focusDefaultWindow (CompScreen *s)
 	if (w && (*w->screen->focusWindow) (w))
 	{
 	    if (!(w->type & (CompWindowTypeDesktopMask |
-			    CompWindowTypeDockMask)))
+			     CompWindowTypeDockMask)))
 		focus = w;
 	}
 	else
@@ -2469,7 +2618,7 @@ focusDefaultWindow (CompScreen *s)
 	    {
 		w = findTopLevelWindowAtDisplay (d, childReturn);
 
-		if (w && (*w->screen->focusWindow) (w))
+		if (w && (*s->focusWindow) (w))
 		{
 		    if (!(w->type & (CompWindowTypeDesktopMask |
 				     CompWindowTypeDockMask)))
@@ -2486,19 +2635,20 @@ focusDefaultWindow (CompScreen *s)
 	    if (w->type & CompWindowTypeDockMask)
 		continue;
 
-	    if ((*s->focusWindow) (w))
+	    if (!(*s->focusWindow) (w))
+		continue;
+
+	    if (!focus)
 	    {
-		if (focus)
-		{
-		    if (w->type & (CompWindowTypeNormalMask |
-				   CompWindowTypeDialogMask |
-				   CompWindowTypeModalDialogMask))
-		    {
-			if (compareWindowActiveness (focus, w) < 0)
-			    focus = w;
-		    }
-		}
-		else
+		focus = w;
+		continue;
+	    }
+
+	    if (w->type & (CompWindowTypeNormalMask |
+			   CompWindowTypeDialogMask |
+			   CompWindowTypeModalDialogMask))
+	    {
+		if (compareWindowActiveness (focus, w) < 0)
 		    focus = w;
 	    }
 	}
@@ -3132,66 +3282,73 @@ computeWorkareaForBox (CompScreen *s,
 	if (!w->mapNum)
 	    continue;
 
-	if (w->struts)
+	if (!w->struts)
+	    continue;
+
+	r.extents.y1 = pBox->y1;
+	r.extents.y2 = pBox->y2;
+
+	x1 = w->struts->left.x;
+	y1 = w->struts->left.y;
+	x2 = x1 + w->struts->left.width;
+	y2 = y1 + w->struts->left.height;
+
+	if (y1 < pBox->y2 && y2 > pBox->y1)
 	{
-	    r.extents.y1 = pBox->y1;
-	    r.extents.y2 = pBox->y2;
+	    r.extents.x1 = x1;
+	    r.extents.x2 = x2;
 
-	    x1 = w->struts->left.x;
-	    y1 = w->struts->left.y;
-	    x2 = x1 + w->struts->left.width;
-	    y2 = y1 + w->struts->left.height;
-
-	    if (y1 < pBox->y2 && y2 > pBox->y1)
-	    {
-		r.extents.x1 = x1;
-		r.extents.x2 = x2;
-
-		XSubtractRegion (region, &r, region);
-	    }
-
-	    x1 = w->struts->right.x;
-	    y1 = w->struts->right.y;
-	    x2 = x1 + w->struts->right.width;
-	    y2 = y1 + w->struts->right.height;
-
-	    if (y1 < pBox->y2 && y2 > pBox->y1)
-	    {
-		r.extents.x1 = x1;
-		r.extents.x2 = x2;
-
-		XSubtractRegion (region, &r, region);
-	    }
-
-	    r.extents.x1 = pBox->x1;
-	    r.extents.x2 = pBox->x2;
-
-	    x1 = w->struts->top.x;
-	    y1 = w->struts->top.y;
-	    x2 = x1 + w->struts->top.width;
-	    y2 = y1 + w->struts->top.height;
-
-	    if (x1 < pBox->x2 && x2 > pBox->x1)
-	    {
-		r.extents.y1 = y1;
-		r.extents.y2 = y2;
-
-		XSubtractRegion (region, &r, region);
-	    }
-
-	    x1 = w->struts->bottom.x;
-	    y1 = w->struts->bottom.y;
-	    x2 = x1 + w->struts->bottom.width;
-	    y2 = y1 + w->struts->bottom.height;
-
-	    if (x1 < pBox->x2 && x2 > pBox->x1)
-	    {
-		r.extents.y1 = y1;
-		r.extents.y2 = y2;
-
-		XSubtractRegion (region, &r, region);
-	    }
+	    XSubtractRegion (region, &r, region);
 	}
+
+	x1 = w->struts->right.x;
+	y1 = w->struts->right.y;
+	x2 = x1 + w->struts->right.width;
+	y2 = y1 + w->struts->right.height;
+
+	if (y1 < pBox->y2 && y2 > pBox->y1)
+	{
+	    r.extents.x1 = x1;
+	    r.extents.x2 = x2;
+
+	    XSubtractRegion (region, &r, region);
+	}
+
+	r.extents.x1 = pBox->x1;
+	r.extents.x2 = pBox->x2;
+
+	x1 = w->struts->top.x;
+	y1 = w->struts->top.y;
+	x2 = x1 + w->struts->top.width;
+	y2 = y1 + w->struts->top.height;
+
+	if (x1 < pBox->x2 && x2 > pBox->x1)
+	{
+	    r.extents.y1 = y1;
+	    r.extents.y2 = y2;
+
+	    XSubtractRegion (region, &r, region);
+	}
+
+	x1 = w->struts->bottom.x;
+	y1 = w->struts->bottom.y;
+	x2 = x1 + w->struts->bottom.width;
+	y2 = y1 + w->struts->bottom.height;
+
+	if (x1 < pBox->x2 && x2 > pBox->x1)
+	{
+	    r.extents.y1 = y1;
+	    r.extents.y2 = y2;
+
+	    XSubtractRegion (region, &r, region);
+	}
+    }
+
+    if (XEmptyRegion (region))
+    {
+	compLogMessage ("core", CompLogLevelWarn,
+			"Empty box after applying struts, ignoring struts");
+	region->extents = *pBox;
     }
 
     area->x      = region->extents.x1;
@@ -3406,9 +3563,10 @@ getActiveWindow (CompDisplay *display,
 				 XA_WINDOW, &actual, &format,
 				 &n, &left, &data);
 
-    if (result == Success && n && data)
+    if (result == Success && data)
     {
-	memcpy (&w, data, sizeof (Window));
+	if (n)
+	    memcpy (&w, data, sizeof (Window));
 	XFree (data);
     }
 
@@ -3724,7 +3882,7 @@ applyStartupProperties (CompScreen *screen,
 
 	workspace = sn_startup_sequence_get_workspace (s->sequence);
 	if (workspace >= 0)
-	    window->desktop = workspace;
+	    setDesktopForWindow (window, workspace);
 
 	window->initialTimestamp    =
 	    sn_startup_sequence_get_timestamp (s->sequence);
@@ -4002,8 +4160,7 @@ clearScreenOutput (CompScreen	*s,
    more than one viewport the most appropriate viewport is returned. How the
    most appropriate viewport is computed can be made optional if necessary. It
    is currently computed as the viewport where the center of the window is
-   located, except for when the window is visible in the current viewport as
-   the current viewport is then always returned. */
+   located. */
 void
 viewportForGeometry (CompScreen *s,
 		     int	x,
@@ -4020,18 +4177,6 @@ viewportForGeometry (CompScreen *s,
     width  += borderWidth * 2;
     height += borderWidth * 2;
 
-    if ((x < s->width  && x + width  > 0) &&
-	(y < s->height && y + height > 0))
-    {
-	if (viewportX)
-	    *viewportX = s->x;
-
-	if (viewportY)
-	    *viewportY = s->y;
-
-	return;
-    }
-
     if (viewportX)
     {
 	centerX = x + (width >> 1);
@@ -4092,17 +4237,26 @@ outputDeviceForGeometry (CompScreen *s,
 
     if (strategy == OUTPUT_OVERLAP_MODE_SMART)
     {
+	int centerX, centerY;
+
 	/* for smart mode, calculate the overlap of the whole rectangle
 	   with the output device rectangle */
 	geomRect.x2 = width + 2 * borderWidth;
 	geomRect.y2 = height + 2 * borderWidth;
 
 	geomRect.x1 = x % s->width;
-	if ((geomRect.x1 + geomRect.x2 / 2) < 0)
+	centerX = geomRect.x1 + (geomRect.x2 / 2);
+	if (centerX < 0)
 	    geomRect.x1 += s->width;
+	else if (centerX > s->width)
+	    geomRect.x1 -= s->width;
+
 	geomRect.y1 = y % s->height;
-	if ((geomRect.y1 + geomRect.y2 / 2) < 0)
+	centerY = geomRect.y1 + (geomRect.y2 / 2);
+	if (centerY < 0)
 	    geomRect.y1 += s->height;
+	else if (centerY > s->height)
+	    geomRect.y1 -= s->height;
 
 	geomRect.x2 += geomRect.x1;
 	geomRect.y2 += geomRect.y1;
@@ -4173,7 +4327,7 @@ outputDeviceForGeometry (CompScreen *s,
 		}
 	    }
     }
-    
+
     return highest;
 }
 
